from sqlalchemy import *
from migrate import *
from dictionaries.versions import views_helper
from datetime import datetime
try:
    from sqlalchemy.dialects.mssql.pyodbc import MSDialect_pyodbc
except:
    from sqlalchemy.databases.mssql import MSSQLDialect_pyodbc as MSDialect_pyodbc

def getPricesTable(meta):
    prices1 = Table('prices1', meta, autoload=True)
    prices2 = Table('prices2', meta, autoload=True)
    return [prices1, prices2]

def getTpricesTable(meta):
    tprices1 = Table('tprices1', meta, autoload=True)
    tprices2 = Table('tprices2', meta, autoload=True)
    return [tprices1, tprices2]

def upgrade(migrate_engine):
    #meta = MetaData(bind = migrate_engine)
    meta = MetaData(bind=migrate_engine)
    for tbl in getPricesTable(meta):
        if not 'minprice' in tbl.columns:
            Column('minprice', Numeric(precision=15,scale=2), default=0.0, nullable=True).create(tbl)
        if 'minprice' in tbl.columns:
            tbl.c.minprice.alter(nullable=False)
    for tbl in getTpricesTable(meta):
        if not 'documentid' in tbl.columns:
            Column('documentid',String(100), default='1', nullable=True).create(tbl)
        if not 'effectivedate' in tbl.columns:
            Column('effectivedate',DateTime, default=datetime(1970,1,1), nullable=True).create(tbl)
        if 'documentid' in tbl.columns:
            tbl.c.documentid.alter(nullable=False)
        if 'effectivedate' in tbl.columns:
            tbl.c.effectivedate.alter(nullable=False)
        if not 'tpricesidxdocumentid' in [ ind.name for ind in tbl.indexes ] or type(migrate_engine.connect().dialect) is MSDialect_pyodbc:
            Index('tpricesidxdocumentid', tbl.c.tindex, tbl.c.bcode, tbl.c.documentid, unique=True).create()
    views_helper.update_dic_view(migrate_engine, 'prices')
    views_helper.update_dic_view(migrate_engine, 'tprices')

def downgrade(migrate_engine):
    #meta = MetaData(bind = migrate_engine)
    meta = MetaData(bind=migrate_engine)
    pricesTblname = views_helper.get_table_name(migrate_engine,'prices')
    for tbl in getPricesTable(meta):
        if 'minprice' in tbl.columns:
            tbl.c.minprice.drop()
    tpricesTblname = views_helper.get_table_name(migrate_engine,'tprices')
    for tbl in getTpricesTable(meta):
        if 'tpricesidxdocumentid' in [ ind.name for ind in tbl.indexes ] or type(migrate_engine.connect().dialect) is MSDialect_pyodbc:
            Index('tpricesidxdocumentid', tbl.c.tindex, tbl.c.bcode, tbl.c.documentid, unique=True).drop()
        if 'documentid' in tbl.columns:
            tbl.c.documentid.drop()
        if 'effectivedate' in tbl.columns:
            tbl.c.effectivedate.drop()
    views_helper.update_dic_view(migrate_engine, 'prices', pricesTblname)
    views_helper.update_dic_view(migrate_engine, 'tprices', tpricesTblname)
